with Ada.Command_Line;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Text_IO;use Ada.Text_IO;
with Ada.Strings.Unbounded.Text_IO; use Ada.Strings.Unbounded.Text_IO;
with Ada.Numerics.Elementary_Functions; use Ada.Numerics.Elementary_Functions;
with Ada.Numerics.Discrete_Random;
with Ada.Numerics.Float_Random;

with BoundedBuffers;

procedure BBtest is

subtype Rand_Range is Integer range 1..113;
package Int_Rand is new Ada.Numerics.Discrete_Random(Rand_Range);

subtype Float_Range is Integer range 1..1153;
package Float_Rand is new Ada.Numerics.Discrete_Random(Float_Range);


-- BB_Capacity : constant Positive:= 16;
type BB_Capacity is mod 64;
package Int_BBuffers is new BoundedBuffers(Integer,BB_Capacity); use Int_BBuffers;

subtype Test_Range is Integer range 0..39;
package Test_Rand is new Ada.Numerics.Discrete_Random(Test_Range);

Shared: BBuffer;

task type Consumer is
   entry Total(Number: out Natural);
end Consumer;

task body Consumer is
   Consumed_Total: Natural:= 0;
   G_Delay: Float_Rand.Generator;
   Item:Integer;
begin
   Float_Rand.Reset(G_Delay,13);
   loop
      select
         accept Total (Number: out Natural) do
            Number:= Consumed_Total;
         end Total;
      or
         delay Duration(Log(Float(Float_Rand.Random(G_Delay)) + Float'Model_Small)/5.0);
      end select;
      Shared.Get(Item);
      Put_Line("Consumed:"&Integer'Image(Item));
      Consumed_Total:= Consumed_Total+1;
   end loop;
end Consumer;

task type Producer is
   entry Total(Number: out Natural);
end Producer;

task body Producer is
   Produced_Total: Natural:= 0;
   G_Delay: Float_Rand.Generator;
   G_Item: Int_Rand.Generator;
   Item:Integer;
begin
   Float_Rand.Reset(G_Delay,17);
   Int_Rand.Reset(G_Item);
   loop
      select
         accept Total (Number: out Natural) do
            Number:= Produced_Total;
         end Total;
      or
         delay Duration(Log(Float(Float_Rand.Random(G_Delay)) + Float'Model_Small)/7.0);
      end select;
      Item:= Int_Rand.Random(G_Item);
      Shared.Put(Item);
      Put_Line("Produced:"&Integer'Image(Item));
      Produced_Total:= Produced_Total+1;
   end loop;
end Producer;


   P: array (Natural range 0..4) of Producer;
   C: array(Natural range 0..7) of Consumer;
   G: Test_Rand.Generator;
   T,Val: Integer;
begin
   Test_Rand.Reset(G);
   for I in 1..100 loop
      T:= Test_Rand.Random(G);
      if (P'First <= T and T<=P'Last) then
         P(T).Total(Val);
         Put_Line("Producer "&Integer'Image(T)&" : "&Integer'Image(Val));
      else
         T:=T-P'Length;
         if (C'First <= T and T <= C'Last) then
            C(T).Total(Val);
            Put_Line("Consumer "&Integer'Image(T)&" : "&Integer'Image(Val));
         end if;
      end if;
      Put_Line("Main : "&Integer'Image(I)&"<===========");
   end loop;
exception
   when others => null;
end BBtest;

